import json
import urllib

import requests

from spotinst import aws_elastigroup


class SpotinstClient:
    __account_id_key = "accountId"
    __base_elastigroup_url = "https://api.spotinst.io/aws/ec2/group"

    def __init__(self, authToken, accountId=None):
        self.authToken = authToken
        self.account_id = accountId

    def create_elastigroup(self, group):

        group = aws_elastigroup.ElastigroupCreationRequest(group)
        excluded_group = self.exclude_missing(json.loads(group.toJSON()))
        body = json.dumps(excluded_group)

        print body
        groupResponse = self.sendPost(body, self.__base_elastigroup_url)

        retVal = groupResponse["response"]["items"][0]

        return retVal

    def update_elastigroup(self, group_update, groupId):

        group = aws_elastigroup.ElastigroupUpdateRequest(group_update)
        excluded_group_update = self.exclude_missing(json.loads(group.toJSON()))
        body = json.dumps(excluded_group_update)

        print body
        groupResponse = self.sendPut(body, self.__base_elastigroup_url + "/" + groupId)

        retVal = groupResponse["response"]["items"][0]

        return retVal

    def delete_elastigroup(self, groupId):
        delurl = self.__base_elastigroup_url + "/" + groupId
        response = self.sendDelete(url=delurl)
        return response

    def get_elastigroup(self, groupId):
        geturl = self.__base_elastigroup_url + "/" + groupId
        result = self.send_get(url=geturl)
        return result["response"]["items"][0]

    def get_elastigroups(self):
        content = self.send_get(url=self.__base_elastigroup_url)
        return content["response"]["items"]

    def send_get(self, url):
        queryParams = None
        if self.account_id is not None:
            queryParams = urllib.urlencode(dict({self.__account_id_key: self.account_id}))

        headers = dict({'Content-Type': 'application/json', 'Authorization': 'Bearer ' + self.authToken})

        print "Sending get request to spotinst API."

        result = requests.get(url, params=queryParams, headers=headers)

        if result.status_code == self.HTTP_STATUS_CODES.SUCCESS:
            data = json.loads(result.content)
            return data
        else:
            print result.status_code
            data = json.loads(result.content)
            return data

    def sendDelete(self, url):
        queryParams = None
        if self.account_id is not None:
            queryParams = urllib.urlencode(dict({self.__account_id_key: self.account_id}))

        headers = dict({'Content-Type': 'application/json', 'Authorization': 'Bearer ' + self.authToken})

        print "Sending deletion request to spotinst API."

        result = requests.delete(url, params=queryParams, headers=headers)

        if result.status_code == self.HTTP_STATUS_CODES.SUCCESS:
            return True
        else:
            print result.status_code
            data = json.loads(result.content)
            print json.dumps(data["response"])
            return False

    def sendPost(self, body, url):
        queryParams = None
        if self.account_id is not None:
            queryParams = urllib.urlencode(dict({self.__account_id_key: self.account_id}))

        headers = dict({'Content-Type': 'application/json', 'Authorization': 'Bearer ' + self.authToken})

        print "Sending post request to spotinst API."

        result = requests.post(url, params=queryParams, data=body, headers=headers)

        if result.status_code == self.HTTP_STATUS_CODES.SUCCESS:
            print "Success"
            data = json.loads(result.content)
        else:
            print result.status_code
            data = json.loads(result.content)
            print json.dumps(data["response"])

        return data

    def sendPut(self, body, url):
        query_params = None
        if self.account_id is not None:
            query_params = urllib.urlencode(dict({self.__account_id_key: self.account_id}))

        headers = dict({'Content-Type': 'application/json', 'Authorization': 'Bearer ' + self.authToken})

        print "Sending put request to spotinst API."

        result = requests.put(url, params=query_params, data=body, headers=headers)

        if result.status_code == self.HTTP_STATUS_CODES.SUCCESS:
            print "Success"
            data = json.loads(result.content)
        else:
            print result.status_code
            data = json.loads(result.content)
            print json.dumps(data["response"])

        return data

    class HTTP_STATUS_CODES:
        SUCCESS = 200
        BAD_REQUEST = 400
        UNAUTHORIZED = 401
        INTERNAL_SERVER_ERROR = 500

    def exclude_missing(self, obj):
        # Delete keys with the value 'none' in a dictionary, recursively.

        # if obj.items() is not None:
        if obj.items() is not None:
            for key, value in obj.items():

                # Remove none values
                if value == aws_elastigroup.none:
                    del obj[key]

                # Handle Objects
                elif isinstance(value, dict):
                    self.exclude_missing(obj=value)

                # Handle lists
                elif self.is_sequence(arg=value):
                    for listitem in value:
                        # Handle Lists of objects
                        try:
                            self.exclude_missing(obj=listitem)
                        except AttributeError:
                            pass
        return obj  # For convenience

    def is_sequence(self, arg):
        return (not hasattr(arg, "strip") and
                hasattr(arg, "__getitem__") or
                hasattr(arg, "__iter__"))



# Always prefer setuptools over distutils
from codecs import open
from os import path

from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='Spotinst',

    version='1.0.5',

    description='A Python SDK for Spotinst',
    long_description='This SDK will allow you to manage your resources in Spotinst easily with Python',

    # The project's main homepage.
    url='https://github.com/spotinst/spotinst-sdk-python',

    # Author details
    author='Spotinst',
    author_email='service@spotinst.com',

    # Choose your license
    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7'
    ],

    keywords='spotinst spot instances aws ec2 cloud infrastructure development elastigroup',
    packages=["spotinst"],
    install_requires=['requests']
)



import json

none = "d3043820717d74d9a17694c176d39733"


# region Elastigroup
class Elastigroup:
    def __init__(self, name=none, description=none, capacity=none, strategy=none,
                 compute=none, scaling=none,
                 scheduling=none, multai=none,
                 thirdPartiesIntegration=none):
        self.name = name
        self.description = description
        self.capacity = capacity
        self.strategy = strategy
        self.scaling = scaling
        self.scheduling = scheduling
        self.multai = multai
        self.thirdPartiesIntegration = thirdPartiesIntegration
        self.compute = compute


# endregion

# region Strategy
class Strategy:
    def __init__(self, availabilityVsCost=none, risk=none, utilizeReservedInstances=none,
                 fallbackToOd=none,
                 onDemandCount=none, drainingTimeout=none,
                 spinUpTime=none, lifetimePeriod=none, signalsList=none,
                 scalingStrategy=none, persistence=none):
        self.risk = risk
        self.utilizeReservedInstances = utilizeReservedInstances
        self.fallbackToOd = fallbackToOd
        self.onDemandCount = onDemandCount
        self.availabilityVsCost = availabilityVsCost
        self.drainingTimeout = drainingTimeout
        self.spinUpTime = spinUpTime
        self.lifetimePeriod = lifetimePeriod
        self.spinUpTime = spinUpTime
        self.signals = signalsList
        self.scalingStrategy = scalingStrategy
        self.persistence = persistence


class Signal:
    def __init__(self, name=none, timeout=none):
        self.name = name
        self.timeout = timeout


class ScalingStrategy:
    def __init__(self, terminateAtEndOfBillingHour):
        self.terminateAtEndOfBillingHour = terminateAtEndOfBillingHour


class Persistence:
    def __init__(self, shouldPersistBlockDevices=none, shouldPersistRootDevice=none,
                 shouldPersistPrivateIp=none):
        self.shouldPersistBlockDevices = shouldPersistBlockDevices
        self.shouldPersistRootDevice = shouldPersistRootDevice
        self.shouldPersistPrivateIp = shouldPersistPrivateIp


# endregion

# region Capacity
class Capacity:
    def __init__(self, minimum=none, maximum=none, target=none, unit=none):
        self.minimum = minimum
        self.maximum = maximum
        self.target = target
        self.unit = unit


# endregion

# region Scaling
class Scaling:
    def __init__(self, up=none, down=none):
        self.up = up
        self.down = down


class ScalingPolicyDimension:
    def __init__(self, name=none, value=none):
        self.name = name
        self.value = value


class ScalingPolicyAction:
    def __init__(self, type=none, adjustment=none, minTargetCapacity=none,
                 maxTargetCapacity=none, target=none,
                 minimum=none,
                 maximum=none):
        self.type = type
        self.adjustment = adjustment
        self.minTargetCapacity = minTargetCapacity
        self.maxTargetCapacity = maxTargetCapacity
        self.target = target
        self.minimum = minimum
        self.maximum = maximum


class ScalingPolicy:
    def __init__(self, namespace=none, metricName=none, statistic=none,
                 evaluationPeriods=none, period=none,
                 threshold=none,
                 cooldown=none, action=none, unit=none, operator=none,
                 dimensions=none, policyName=none):
        self.policyName = policyName
        self.namespace = namespace
        self.metricName = metricName
        self.dimensions = dimensions
        self.statistic = statistic
        self.evaluationPeriods = evaluationPeriods
        self.period = period
        self.threshold = threshold
        self.cooldown = cooldown
        self.action = action
        self.unit = unit
        self.operator = operator


# endregion

# region Scheduling
class Scheduling:
    def __init__(self, tasks=none):
        self.tasks = tasks


class ScheduledTask:
    def __init__(self, taskType=none, scaleTargetCapacity=none,
                 scaleMinCapacity=none,
                 scaleMaxCapacity=none, batchSizePercentage=none, gracePeriod=none,
                 adjustment=none,
                 adjustmentPercentage=none, isEnabled=none,
                 frequency=none, cronExpression=none, ):
        self.isEnabled = isEnabled
        self.frequency = frequency
        self.cronExpression = cronExpression
        self.taskType = taskType
        self.scaleTargetCapacity = scaleTargetCapacity
        self.scaleMinCapacity = scaleMinCapacity
        self.scaleMaxCapacity = scaleMaxCapacity
        self.batchSizePercentage = batchSizePercentage
        self.gracePeriod = gracePeriod
        self.adjustment = adjustment
        self.adjustmentPercentage = adjustmentPercentage


# endregion

# region Multai
class Multai:
    def __init__(self, token=none, balancers=none):
        self.token = token
        self.balancers = balancers


class MultaiLoadBalancer:
    def __init__(self, projectId=none, balancerId=none, targetSetId=none, azAwareness=none,
                 autoWeight=none):
        self.projectId = projectId
        self.balancerId = balancerId
        self.targetSetId = targetSetId
        self.azAwareness = azAwareness
        self.autoWeight = autoWeight


# endregion

# region ThirdPartyIntegrations
class Rancher:
    def __init__(self, accessKey=none, secretKey=none, masterHost=none):
        self.accessKey = accessKey
        self.secretKey = secretKey
        self.masterHost = masterHost


class Mesosphere:
    def __init__(self, apiServer=none):
        self.apiServer = apiServer


class ElasticBeanstalk:
    def __init__(self, environmentId=none, deploymentPreferences=none):
        self.environmentId = environmentId
        self.deploymentPreferences = deploymentPreferences


class DeploymentPreferences:
    def __init__(self, automaticRoll=none, batchSizePercentage=none, gracePeriod=none,
                 strategy=none):
        self.automaticRoll = automaticRoll
        self.batchSizePercentage = batchSizePercentage
        self.gracePeriod = gracePeriod
        self.strategy = strategy


class BeanstalkDeploymentStrategy:
    def __init__(self, action=none, shouldDrainInstances=none):
        self.action = action
        self.shouldDrainInstances = shouldDrainInstances


class EcsConfiguration:
    def __init__(self, clusterName=none):
        self.clusterName = clusterName


class KubernetesConfiguration:
    def __init__(self, apiServer=none, token=none):
        self.apiServer = apiServer
        self.token = token


class RightScaleConfiguration:
    def __init__(self, accountId=none, refreshToken=none):
        self.accountId = accountId
        self.refreshToken = refreshToken


class OpsWorksConfiguration:
    def __init__(self, layerId=none):
        self.layerId = layerId


class ChefConfiguration:
    def __init__(self, chefServer=none, organization=none, user=none, pemKey=none,
                 chefVersion=none):
        self.chefServer = chefServer
        self.organization = organization
        self.user = user
        self.pemKey = pemKey
        self.chefVersion = chefVersion


class ThirdPartyIntegrations:
    def __init__(self, rancher=none, mesosphere=none, elasticBeanstalk=none, ecs=none,
                 kubernetes=none, rightScale=none,
                 opsWorks=none, chef=none):
        self.rancher = rancher
        self.mesosphere = mesosphere
        self.elasticBeanstalk = elasticBeanstalk
        self.ecs = ecs
        self.kubernetes = kubernetes
        self.rightScale = rightScale
        self.opsWorks = opsWorks
        self.chef = chef


# endregion

# region Compute
class Compute:
    def __init__(self, launchSpecification=none, instanceTypes=none, product=none,
                 availabilityZonesList=none,
                 elasticIpsStringList=none,
                 ebsVolumesList=none):
        self.elasticIps = elasticIpsStringList
        self.instanceTypes = instanceTypes
        self.availabilityZones = availabilityZonesList
        self.product = product
        self.ebsVolumePool = ebsVolumesList
        self.launchSpecification = launchSpecification


class AvailabilityZone:
    def __init__(self, name=none, subnetId=none, subnetIds=none, placementGroupName=none):
        self.name = name
        self.subnetId = subnetId
        self.subnetIds = subnetIds
        self.placementGroupName = placementGroupName


class InstanceTypes:
    def __init__(self, ondemand=none, spotTypesList=none, instanceTypeWeightsList=none):
        self.ondemand = ondemand
        self.spot = spotTypesList
        self.weights = instanceTypeWeightsList


class Weight:
    def __init__(self, instanceType=none, weightedCapacity=none):
        self.instanceType = instanceType
        self.weightedCapacity = weightedCapacity


class EbsVolume:
    def __init__(self, deviceName=none, volumeIds=none):
        self.deviceName = deviceName
        self.volumeIds = volumeIds


class LaunchSpecification:
    def __init__(self, securityGroupIdsList=none, imageId=none, monitoring=none,
                 healthCheckType=none,
                 loadBalancersConfig=none,
                 healthCheckGracePeriod=none, healthCheckUnhealthyDurationBeforeReplacement=none,
                 ebsOptimized=none, tenancy=none, iamRole=none, keyPair=none,
                 userData=none, shutdownScript=none,
                 blockDeviceMappingsList=none,
                 networkInterfacesList=none, tagsList=none):
        self.loadBalancersConfig = loadBalancersConfig
        self.healthCheckType = healthCheckType
        self.healthCheckGracePeriod = healthCheckGracePeriod
        self.healthCheckUnhealthyDurationBeforeReplacement = healthCheckUnhealthyDurationBeforeReplacement
        self.securityGroupIds = securityGroupIdsList
        self.monitoring = monitoring
        self.ebsOptimized = ebsOptimized
        self.imageId = imageId
        self.tenancy = tenancy
        self.iamRole = iamRole
        self.keyPair = keyPair
        self.userData = userData
        self.shutdownScript = shutdownScript
        self.blockDeviceMappings = blockDeviceMappingsList
        self.networkInterfaces = networkInterfacesList
        self.tags = tagsList


class LoadBalancersConfig:
    def __init__(self, loadBalancersList=none):
        self.loadBalancers = loadBalancersList


class LoadBalancer:
    def __init__(self, type=none, arn=none, name=none):
        self.name = name
        self.arn = arn
        self.type = type


class IAmRole:
    def __init__(self, name=none, arn=none):
        self.name = name
        self.arn = arn


class BlockDeviceMapping:
    def __init__(self, deviceName=none, ebs=none, noDevice=none, virtualName=none):
        self.deviceName = deviceName
        self.ebs = ebs
        self.noDevice = noDevice
        self.virtualName = virtualName


class EBS:
    def __init__(self, deleteOnTermination=none, encrypted=none, iops=none, snapshotId=none,
                 volumeSize=none,
                 volumeType=none):
        self.deleteOnTermination = deleteOnTermination
        self.encrypted = encrypted
        self.iops = iops
        self.snapshotId = snapshotId
        self.volumeSize = volumeSize
        self.volumeType = volumeType


class Tag:
    def __init__(self, tagKey=none, tagValue=none):
        self.tagKey = tagKey
        self.tagValue = tagValue


class NetworkInterface:
    def __init__(self, deleteOnTermination=none, deviceIndex=none, description=none,
                 secondaryPrivateIpAddressCount=none,
                 associatePublicIpAddress=none, groupsList=none, networkInterfaceId=none,
                 privateIpAddress=none,
                 privateIpAddressObjList=none, subnetId=none,
                 associateIpv6Address=none):
        self.description = description
        self.deviceIndex = deviceIndex
        self.secondaryPrivateIpAddressCount = secondaryPrivateIpAddressCount
        self.associatePublicIpAddress = associatePublicIpAddress
        self.deleteOnTermination = deleteOnTermination
        self.groups = groupsList
        self.networkInterfaceId = networkInterfaceId
        self.privateIpAddress = privateIpAddress
        self.privateIpAddresses = privateIpAddressObjList
        self.subnetId = subnetId
        self.associateIpv6Address = associateIpv6Address


class PrivateIpAddress:
    def __init__(self, privateIpAddress=none, primary=none):
        self.privateIpAddress = privateIpAddress
        self.primary = primary


# endregion

class ElastigroupCreationRequest:
    def __init__(self, elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ElastigroupUpdateRequest:
    def __init__(self, elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
